/****************************************************************************
**
** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Nokia Corporation (qt-info@nokia.com)
**
** This file is part of the Qt Mobility Components.
**
** $QT_BEGIN_LICENSE:LGPL$
** Commercial Usage
** Licensees holding valid Qt Commercial licenses may use this file in
** accordance with the Qt Solutions Commercial License Agreement provided
** with the Software or, alternatively, in accordance with the terms
** contained in a written agreement between you and Nokia.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Nokia gives you certain additional
** rights.  These rights are described in the Nokia Qt LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3.0 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU General Public License version 3.0 requirements will be
** met: http://www.gnu.org/copyleft/gpl.html.
**
** Please note Third Party Software included with Qt Solutions may impose
** additional restrictions and it is the user's responsibility to ensure
** that they have met the licensing requirements of the GPL, LGPL, or Qt
** Solutions Commercial license and the relevant license of the Third
** Party Software they are using.
**
** If you are unsure which license is appropriate for your use, please
** contact the sales department at qt-sales@nokia.com.
** $QT_END_LICENSE$
**
****************************************************************************/

#include "qdeclarativegalleryitem.h"

#include <qgalleryresultset.h>

#include <QtDeclarative/qdeclarativepropertymap.h>

QTM_BEGIN_NAMESPACE

QDeclarativeGalleryItem::QDeclarativeGalleryItem(QObject *parent)
    : QObject(parent)
    , m_metaData(0)
    , m_status(Null)
    , m_complete(false)
{
    connect(&m_request, SIGNAL(statusChanged(QGalleryAbstractRequest::Status)),
            this, SLOT(_q_statusChanged()));
    connect(&m_request, SIGNAL(progressChanged(int,int)), this, SIGNAL(progressChanged()));

    connect(&m_request, SIGNAL(itemChanged()),
            this, SLOT(_q_itemChanged()));
    connect(&m_request, SIGNAL(metaDataChanged(QList<int>)),
            this, SLOT(_q_metaDataChanged(QList<int>)));

    m_metaData = new QDeclarativePropertyMap(this);

    connect(m_metaData, SIGNAL(valueChanged(QString,QVariant)),
            this, SLOT(_q_valueChanged(QString,QVariant)));
}

QDeclarativeGalleryItem::~QDeclarativeGalleryItem()
{
}

void QDeclarativeGalleryItem::componentComplete()
{
    m_complete = true;

    if (m_request.itemId().isValid())
        m_request.execute();
}

void QDeclarativeGalleryItem::_q_statusChanged()
{
    Status status = m_status;
    QString message = m_request.errorString();

    m_status = Status(m_request.status());

    qSwap(message, m_errorMessage);

    if (m_status != status) {
        m_status = status;

        emit statusChanged();
    }

    if (message != m_errorMessage)
        emit errorMessageChanged();
}

void QDeclarativeGalleryItem::_q_itemChanged()
{
    if (m_request.isValid()) {
        for (QHash<int, QString>::const_iterator it = m_propertyKeys.constBegin();
                it != m_propertyKeys.constEnd();
                ++it) {
            if (m_request.propertyKey(it.value()) < 0)
                m_metaData->clear(it.value());
        }

        m_propertyKeys.clear();

        const QStringList propertyNames = m_request.propertyNames();

        for (QStringList::const_iterator it = propertyNames.begin(); it != propertyNames.end(); ++it) {
            const int key = m_request.propertyKey(*it);

            if (key >= 0) {
                m_propertyKeys.insert(key, *it);

                QVariant value = m_request.metaData(key);
                m_metaData->insert(*it, value.isNull()
                        ? QVariant(m_request.propertyType(key))
                        : value);
            }
        }
    } else {
        typedef QHash<int, QString>::const_iterator iterator;
        for (iterator it = m_propertyKeys.constBegin(); it != m_propertyKeys.constEnd(); ++it)
            m_metaData->clear(it.value());

        m_propertyKeys.clear();
    }

    emit availableChanged();
}

void QDeclarativeGalleryItem::_q_metaDataChanged(const QList<int> &keys)
{
    typedef QList<int>::const_iterator iterator;
    for (iterator it = keys.begin(); it != keys.end(); ++it){
        QVariant value = m_request.metaData(*it);
        m_metaData->insert(m_propertyKeys.value(*it), value.isNull()
                ? QVariant(m_request.propertyType(*it))
                : value);
    }
}

/*!
    \qmlclass DocumentGalleryItem QDeclarativeDocumentGalleryItem

    \inmodule QtGallery
    \ingroup qml-gallery

    \brief The DocumentGalleryItem element allows you to request information
    about a single item from the document gallery

    This element is part of the \bold {QtMobility.gallery 1.1} module.


    \sa DocumentGalleryModel, DocumentGalleryType
*/

QDeclarativeDocumentGalleryItem::QDeclarativeDocumentGalleryItem(QObject *parent)
    : QDeclarativeGalleryItem(parent)
{
    connect(this, SIGNAL(availableChanged()), this, SIGNAL(itemTypeChanged()));
}

QDeclarativeDocumentGalleryItem::~QDeclarativeDocumentGalleryItem()
{
}

void QDeclarativeDocumentGalleryItem::classBegin()
{
    m_request.setGallery(QDeclarativeDocumentGallery::gallery(this));
}

/*!
    \qmlproperty enum DocumentGalleryItem::status

    This property holds the status of an item request.  It can be one of:

    \list
    \o Null No \l item has been specified.
    \o Active Information about an \l item is being fetched from the gallery.
    \o Finished Information about an \l item is available.
    \o Idle Information about an \l item which will be automatically
    updated is available.
    \o Cancelling The query was cancelled but hasn't yet reached the
    cancelled status.
    \o Cancelled The query was cancelled.
    \o Error Information about a type could not be retrieved due to an error.
    \endlist
*/

/*!
    \qmlproperty real DocumentGalleryItem::progress

    This property holds the current progress of the request, from 0.0 (started)
    to 1.0 (finished).
*/

/*!
    \qmlproperty QStringList DocumentGalleryItem::properties

    This property holds the item properties a request should return values for.
*/

/*!
    \qmlproperty bool DocumentGalleryItem::autoUpdate

    This property holds whether a request should refresh its results
    automatically.
*/

/*!
    \qmlproperty variant DocumentGalleryItem::item

    This property holds the id of the item to return information about.
*/

/*!
    \qmlproperty bool DocumentGalleryItem::available

    This property holds whether the meta-data of an item is available.
*/

/*!
    \qmlproperty enum DocumentGalleryItem::itemType

    This property holds the type of a gallery item. It can be one of:

    \list
    \o DocumentGallery.InvalidType
    \o DocumentGallery.File
    \o DocumentGallery.Folder
    \o DocumentGallery.Document
    \o DocumentGallery.Text
    \o DocumentGallery.Audio
    \o DocumentGallery.Image
    \o DocumentGallery.Video
    \o DocumentGallery.Playlist
    \o DocumentGallery.Artist
    \o DocumentGallery.AlbumArtist
    \o DocumentGallery.Album
    \o DocumentGallery.AudioGenre
    \o DocumentGallery.PhotoAlbum
    \endlist
*/

QDeclarativeDocumentGallery::ItemType QDeclarativeDocumentGalleryItem::itemType() const
{
    return QDeclarativeDocumentGallery::itemTypeFromString(m_request.itemType());
}

/*!
    \qmlproperty url DocumentGalleryItem::itemUrl

    This property holds the URL of a gallery item.
*/

/*!
    \qmlproperty object DocumentGalleryItem::metaData

    This property holds the meta-data of a gallery item.
*/

/*!
    \qmlmethod DocumentGalleryItem::reload()

    Re-queries the gallery.
*/

/*!
    \qmlmethod DocumentGalleryItem::cancel()

    Cancels an executing request.
*/

/*!
    \qmlmethod DocumentGalleryItem::clear()

    Clears the results of a request.
*/

#include "moc_qdeclarativegalleryitem.cpp"

QTM_END_NAMESPACE
